@@ -8,4 +8,10 @@ pub enum CliError {
     UnknownEnvironmentVariable(String),
     #[error("Parameter format invalid")]
     BadParam,
+    #[error("IO error: {0}")]
+    IOError(#[from] std::io::Error),
+    #[error("Serde error: {0}")]
+    SerializationError(#[from] serde_json::Error),
+    #[error("{0}")]
+    LibraryError(#[from] crate::error::Error),
 }
