@@ -4,8 +4,8 @@ pub enum Error {
     HttpRequestError(u16, String),
     #[error("{0}")]
     HttpTransportError(String),
-    #[error("IO error")]
+    #[error("IO error: {0}")]
     IOError(#[from] std::io::Error),
-    #[error("Serde error")]
+    #[error("Serde error: {0}")]
     SerializationError(#[from] serde_json::Error),
 }
