@@ -1,8 +1,9 @@
 #![doc = include_str!("../README.md")]
+#[cfg(feature = "cli")]
 pub mod cli;
 mod config;
 mod error;
-mod metadata;
+mod metadata_api;
 pub mod oauth1;
 mod params;
 mod requester;
@@ -13,3 +14,4 @@ mod suiteql;
 pub use config::*;
 pub use error::*;
 pub use rest_api::*;
+pub use suiteql::*;
