@@ -6,8 +6,8 @@ impl Params {
         Self(Vec::new())
     }
 
-    pub fn push(&mut self, k: String, v: String) {
-        self.0.push((k, v))
+    pub fn push<K: ToString, V: ToString>(&mut self, k: K, v: V) {
+        self.0.push((k.to_string(), v.to_string()))
     }
 
     pub fn get(&self) -> &Vec<(String, String)> {
@@ -27,15 +27,18 @@ impl Default for Params {
     }
 }
 
+#[cfg(feature = "cli")]
 #[derive(Debug)]
 pub(crate) struct ParamStr(String, String);
 
+#[cfg(feature = "cli")]
 impl ParamStr {
     pub(crate) fn new(k: String, v: String) -> Self {
         Self(k, v)
     }
 }
 
+#[cfg(feature = "cli")]
 impl From<Vec<ParamStr>> for Params {
     fn from(param_strings: Vec<ParamStr>) -> Self {
         let mut p = Params::new();
