@@ -1,16 +1,20 @@
+use crate::oauth1::Algorithm;
 use crate::response::Response;
 
 use super::config::Config;
 use super::error::Error;
+use super::metadata_api::MetadataApi;
 use super::params::Params;
 use super::requester::Requester;
 use super::suiteql::SuiteQl;
 
 static DEFAULT_BASE_URL: &str = "https://{}.suitetalk.api.netsuite.com/services/rest";
 
+#[derive(Debug, Clone)]
 pub struct RestApi {
     requester: Requester,
     pub suiteql: SuiteQl,
+    pub metadata: MetadataApi,
 }
 
 impl RestApi {
@@ -18,7 +22,12 @@ impl RestApi {
         let base_url = Self::default_base_url(&config);
         let requester = Requester::new(config, base_url);
         let suiteql = SuiteQl::new(requester.clone());
-        Self { requester, suiteql }
+        let metadata = MetadataApi::new(requester.clone());
+        Self {
+            requester,
+            suiteql,
+            metadata,
+        }
     }
 
     fn default_base_url(config: &Config) -> String {
@@ -26,10 +35,26 @@ impl RestApi {
         DEFAULT_BASE_URL.replace("{}", &host_part)
     }
 
-    pub fn with_base_url(config: Config, base_url: String) -> Self {
-        let requester = Requester::new(config, base_url);
+    pub fn with_base_url(self, base_url: String) -> Self {
+        let requester = self.requester.with_base_url(base_url);
         let suiteql = SuiteQl::new(requester.clone());
-        Self { requester, suiteql }
+        let metadata = MetadataApi::new(requester.clone());
+        Self {
+            requester,
+            suiteql,
+            metadata,
+        }
+    }
+
+    pub fn with_algorithm(self, algorithm: Algorithm) -> Self {
+        let requester = self.requester.with_algorithm(algorithm);
+        let suiteql = SuiteQl::new(requester.clone());
+        let metadata = MetadataApi::new(requester.clone());
+        Self {
+            requester,
+            suiteql,
+            metadata,
+        }
     }
 
     pub fn get_raw(
